package com.webtako.flash {
	import flash.display.Graphics;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.utils.getTimer;
	
	//countdown clock class
	public class Clock extends Sprite {
		public static const TRANSITION_TYPE:String = "easeOutQuart";
		public static const TRANSITION_TIME:Number = 1.0;
		public static const BORDER_SIZE:uint = 1;
		public static const TWO_PI:Number = 2 * Math.PI;
		
		private var _radius:Number;
		private var _color:uint;
   		private var _x1:Number;
   		private var _y1:Number;
   		private var _startTime:uint;
   		private var _elapseTime:uint;
   		private var _timeSpan:uint;
   		private var _g:Graphics;
   		private var _container:Sprite;
   		private var _extendedRadius:Number;
   		
		public function Clock(timeSpan:uint, radius:Number, color:uint = 0xFFFFFF, bgColor:uint = 0x000000, bgAlpha:Number = 0.5) {
			//set properties
			this._radius = radius;
			this._extendedRadius = this._radius + 10;
			this._color = color;							
			this._elapseTime = 0;
			this._x1 = Math.cos(0) * this._extendedRadius;
			this._y1 = Math.sin(0) * this._extendedRadius;
			this._timeSpan = timeSpan;
			
			//init background
			this.graphics.beginFill(bgColor, bgAlpha);
			this.graphics.drawCircle(0, 0, this._radius);
			this.graphics.endFill();	
   			
			//init container 
			this._container = new Sprite();			
			this._g = this._container.graphics;	
			this._g.beginFill(0x000000, 0);
			this._g.drawCircle(0, 0, this._radius);
			this._g.endFill();	
			this._container.rotation = 270;			
			this.initMask(this._container, this._radius - BORDER_SIZE);			
			this.addChild(this._container);
   						
			//init mouse behavior			
			this.useHandCursor = false;	
			this.mouseEnabled = false;
			this.mouseChildren = false;
		}
		
		//get elapse time
		public function get elapseTime():uint {
			return this._elapseTime;	
		}
		
		//get radius
		public function get radius():Number {
			return this._radius;	
		}
		
		//start clock
		public function start():void {
			this._startTime = getTimer();
			this.addEventListener(Event.ENTER_FRAME, update);
		}
		
		//resume clock
		public function resume():void {
			this._startTime = getTimer() - this._elapseTime;
			this.addEventListener(Event.ENTER_FRAME, update);	
		}
		
		//stop/pause clock
		public function stop():void {
			this.removeEventListener(Event.ENTER_FRAME, update);		
			this.reset();				
		}
		
		public function pause():void {
			this.removeEventListener(Event.ENTER_FRAME, update);			
		}
		
		//reset clock
		private function reset():void {
			this._g.clear();
			this._x1 = Math.cos(0) * this._extendedRadius;
			this._y1 = Math.sin(0) * this._extendedRadius;			
			this._elapseTime = 0;	
		}
		
		//update clock
		private function update(event:Event):void {		
			this._elapseTime = getTimer() - this._startTime;
			var angle:Number = (this._elapseTime/this._timeSpan * TWO_PI);	

			var x2:Number = Math.round(Math.cos(angle) * this._extendedRadius);
			var y2:Number = Math.round(Math.sin(angle) * this._extendedRadius);
			
			this._g.moveTo(0, 0);
   			this._g.beginFill(this._color);		
   			this._g.lineTo(this._x1, this._y1);
   			this._g.lineTo(x2, y2);
   			this._g.endFill();
   			
			this._x1 = x2;
			this._y1 = y2;
		}
		
		//init mask	
		private function initMask(obj:Sprite, radius:Number):void {
			var maskSprite:Sprite = new Sprite();
			maskSprite.graphics.beginFill(0x000000);
			maskSprite.graphics.drawCircle(0, 0, radius);
			maskSprite.graphics.endFill();
			
 			obj.mask = maskSprite;
   			obj.addChild(maskSprite);
		}
	}
}