package com.webtako.flash {
	import caurina.transitions.Tweener;
	import caurina.transitions.properties.ColorShortcuts;
	ColorShortcuts.init();
		
	import flash.display.MovieClip;
	import flash.geom.ColorTransform;
	import flash.events.MouseEvent;
	
	public class PlayPauseButton extends MovieClip {		
		public static const TRANSITION_TYPE:String = "easeOutQuart";
		public static const TRANSITION_TIME:Number = 1.0;
		public static const SQUARE:String = "SQUARE";
		public static const CIRCLE:String = "CIRCLE";
		public static const ROUND_SQUARE:String = "ROUND_SQUARE";
		public static const DEFAULT_SIZE:Number = 28;
		
		private var _playMode:Boolean;
		private var _color:uint;
		private var _mouseoverColor:uint;
		private var _container:MovieClip;

		public function PlayPauseButton(playMode:Boolean = false, 
								color:uint = 0xFFFFFF, mouseoverColor:uint = 0x0066FF, 
								bgColor:uint = 0x000000, bgAlpha:Number = 0.5, 
								buttonSize:Number = DEFAULT_SIZE, containerType:String = CIRCLE) {
			//assign properties			
			this._playMode = playMode;						
			this._color = color;
			this._mouseoverColor = mouseoverColor;
			
			//set size
			var scalePct = (buttonSize/this.width);  
			this.scaleX = scalePct;
			this.scaleY = scalePct; 
			
			//set container type
			containerType = containerType.toUpperCase();
			switch (containerType) {
				case SQUARE:
					this._container = this.square;
					this.rsquare.visible = false;
					this.circle.visible = false;
					break;
				case ROUND_SQUARE:
					this._container = this.rsquare;
					this.square.visible = false;
					this.circle.visible = false;
					break;
				default:
					this._container = this.circle;
					this.square.visible = false;
					this.rsquare.visible = false;
			}
			
			//set background color		
			this.setColor(this._container, bgColor);
			this._container.alpha = bgAlpha;
			
			//set play symbol color
			this.setColor(this.playSymbol, this._color);

			//set pause symbol color			
			this.setColor(this.pauseSymbol, this._color);
			
			if (this._playMode) {
				this.pauseSymbol.visible = false;	
			}	
			else {
				this.playSymbol.visible = false;				
			}	
				
			//init mouse behavior			
			this.buttonMode = true;
			this.useHandCursor = true;	
			this.mouseChildren = false;
			
			this.addEventListener(MouseEvent.MOUSE_OVER, onButtonOver);
			this.addEventListener(MouseEvent.MOUSE_OUT, onButtonOut);
		}
		
		//change display symbol
		public function toggleDisplay():void {
			if (this._playMode) {
				this.playSymbol.visible = false;
				this.pauseSymbol.visible = true;
				this.setColor(this.pauseSymbol, this._mouseoverColor);
			}	
			else {
				this.playSymbol.visible = true;
				this.pauseSymbol.visible = false;
				this.setColor(this.playSymbol, this._mouseoverColor);
			}
			this._playMode = !this._playMode;
		}
		
		//is playMode
		public function get playMode():Boolean {
			return this._playMode;	
		}
		
		//is pauseMode
		public function get pauseMode():Boolean {
			return !this._playMode;	
		}
		
		//on mouseover event handler
		private function onButtonOver(event:MouseEvent):void {
			if (this._playMode) {
				Tweener.addTween(this.playSymbol, {_color:this._mouseoverColor, time:TRANSITION_TIME, transition:TRANSITION_TYPE});	
			}
			else {
				Tweener.addTween(this.pauseSymbol, {_color:this._mouseoverColor, time:TRANSITION_TIME, transition:TRANSITION_TYPE});					
			}
		}
		
		//on mouseout event handler
		private function onButtonOut(event:MouseEvent):void {
			if (this._playMode) {
				Tweener.addTween(this.playSymbol, {_color:this._color, time:TRANSITION_TIME, transition:TRANSITION_TYPE});			
			}
			else {
				Tweener.addTween(this.pauseSymbol, {_color:this._color, time:TRANSITION_TIME, transition:TRANSITION_TYPE});					
			}		
		}
		
		private function setColor(mc:MovieClip, color:uint):void {
			var colorTransform:ColorTransform = mc.transform.colorTransform;
			colorTransform.color = color;
			mc.transform.colorTransform = colorTransform;	
		}
	}
}