package com.webtako.flash {
	import caurina.transitions.Tweener;
	
	import flash.display.DisplayObject;
	import flash.display.Loader;
	import flash.display.LoaderInfo;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.MouseEvent;
	import flash.net.URLRequest;
	import flash.net.navigateToURL;
	
	public class Slide extends Sprite {
		public static const TRANSITION_TYPE:String = "easeOutQuart";
		public static const TRANSITION_TIME:Number = 1.0;
		public static const FADE_TRANSITION:String = "FADE";
		public static const SLIDE_TRANSITION:String = "SLIDE"		
		public static const CONTENTS_LOADED:String = "contentsLoaded";
		public static const CONTENT_CHANGE:String = "contentChange";
		public static const SWF_TYPE:String = "application/x-shockwave-flash";
		
		private var _slideWidth:Number;
		private var _slideHeight:Number;	
		private var _link:String;
		private var _target:String;
		private var _paths:Array;
		private var _contents:Array;
		private var _contentStrip:Sprite;
		private var _content:DisplayObject;
		private var _prevContent:DisplayObject;
		private var _maskSprite:Sprite;
		private var _preloader:Preloader;		
		private var _transitionType:String;
		private var _loadIndex:uint;
		private var _bgColor:uint;
			
		public function Slide(transitionType:String, slideWidth:Number, slideHeight:Number,
								bgColor:uint = 0x000000, preloaderColor:uint = 0x0066FF, preloaderSize:Number = 28,
								preloaderTextSize:Number = 10, preloaderTextColor:uint = 0xFFFFFF) {										
			//set properties
			this._slideWidth = slideWidth;
			this._slideHeight = slideHeight;
			this._link = "";
			this._target = "";
			this._paths = new Array();
			this._contents = new Array();
			this._transitionType = transitionType.toUpperCase();			
			this._bgColor = bgColor;
			
			//init background
			this.graphics.beginFill(this._bgColor);
			this.graphics.drawRect(0, 0, this._slideWidth, this._slideHeight);
			this.graphics.endFill();
			
			//init preloader	
			this.initPreloader(preloaderColor, preloaderSize, preloaderTextSize, preloaderTextColor);
		
			//init mask							
			this.initMask();
			
			//init mouse behaviors
			this.buttonMode = true;	
			this.mouseEnabled = true;	
			this.mouseChildren = true;			
			this.useHandCursor = true;	
							
			this.addEventListener(MouseEvent.CLICK, onSlideClick);
		}
		
		//init mask sprite
		private function initMask():void {
			this._maskSprite = new Sprite();			
			this._maskSprite.graphics.beginFill(0x000000);
			this._maskSprite.graphics.drawRect(0, 0, this._slideWidth, this._slideHeight);
			this._maskSprite.graphics.endFill();
			
			this.mask = this._maskSprite;	
			this.addChild(this._maskSprite);
		}
		
		private function initSlideContainer(content:DisplayObject):Sprite {
			var slide:Sprite = new Sprite();
			slide.graphics.beginFill(0x000000, 0);
			slide.graphics.drawRect(0, 0, this._slideWidth, this._slideHeight);
			slide.graphics.endFill();

			content.x = Math.round((this._slideWidth - content.width)/2);
			content.y = Math.round((this._slideHeight - content.height)/2);
			slide.addChild(content);			
	
			var maskSprite:Sprite = new Sprite();			
			maskSprite.graphics.beginFill(0x000000);
			maskSprite.graphics.drawRect(0, 0, this._slideWidth, this._slideHeight);
			maskSprite.graphics.endFill();			
			slide.mask = maskSprite;	
			slide.addChild(maskSprite);	
			
			return slide;
		}
		
		//init preloader
		private function initPreloader(color:uint, size:Number, textSize:Number, textColor:uint):void {
			this._preloader = new Preloader(this._slideWidth, this._slideHeight, size, color, textSize, textColor);
			this.addChild(this._preloader);
		}
		
		//init content strip for sliding transition
		private function initContentStrip():void {
			var stripWidth:Number = this._slideWidth * this._contents.length;
			this._contentStrip = new Sprite();
			this._contentStrip.graphics.beginFill(0x000000, 0);
			this._contentStrip.graphics.drawRect(0, 0, stripWidth, this._slideHeight);
			this._contentStrip.graphics.endFill();
			
			for (var i:uint = 0; i < this._contents.length; i++) {
				var content:DisplayObject = this._contents[i];
				var slide:Sprite = this.initSlideContainer(content);	
				slide.x = i * this._slideWidth;
				this._contentStrip.addChild(slide);									
			}
			this._contentStrip.alpha = 0;
			this.addChild(this._contentStrip);
			
			Tweener.addTween(this._contentStrip, {alpha:1, time:TRANSITION_TIME, transition:TRANSITION_TYPE});		
		}
		
		//init load all content
		public function loadAllContent():void {
			this._loadIndex = 0;
			if (this._loadIndex < this._paths.length) {
				this.loadContent();
			}
		}

		//on load complete event handler
		private function contentComplete(event:Event):void {	
			//add to content list		
			var loaderInfo:LoaderInfo = LoaderInfo(event.currentTarget);
			var content:* = loaderInfo.content;			
			if (loaderInfo.contentType == SWF_TYPE) {
				try {
					content.stop();		
				}
				catch (e:Error) {
					trace(e.message);
				}
 			}
 			this._contents.push(content);	

			this._loadIndex++;
			if (this._loadIndex < this._paths.length) {	
				//load next
				this.loadContent();
			}
			else {										
				//all complete
				this.setComplete();
			}
		}
		
		//content load helper
		private function loadContent():void {
			this._preloader.message = "loading " + (this._loadIndex + 1) + " / " + this._paths.length;
			
			var loader:Loader = new Loader();
			loader.contentLoaderInfo.addEventListener(Event.COMPLETE, contentComplete);
			loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, ioError);
			loader.load(new URLRequest(this._paths[this._loadIndex]));	
		}
		
		//init complete loading
		private function setComplete():void {
			this._paths = null;
			
			//init slide strip for sliding transition
			if (this._transitionType == SLIDE_TRANSITION) {
				this.initContentStrip();
			}
			
			//set preloader to complete
			this._preloader.setComplete();				
			Tweener.addTween(this._preloader, {alpha:0, time:TRANSITION_TIME, transition:TRANSITION_TYPE});
			
			//dispatch event
			dispatchEvent(new Event(CONTENTS_LOADED));			
		}
		
		//display content
		public function displayContent(index:uint):void {
			if (this._transitionType == FADE_TRANSITION) {
				this.fadeInContent(index);
			}	
			else {
				this.slideInContent(index);
			}
		}
		
		//display fade in content
		private function fadeInContent(index:uint):void {
			var oldContent:* = this._prevContent;
			this._prevContent = this._content;
			
			if (oldContent && this.contains(oldContent)) {
				this.removeChild(oldContent);		
			}	
		
			this._content = this._contents[index];
			
			//add content			 
			this._content.alpha = 0;
			this._content.x = Math.round((this._slideWidth - this._content.width)/2);
			this._content.y = Math.round((this._slideHeight - this._content.height)/2);
			this.addChild(this._content);
			
			//fade in/out
			Tweener.addTween(this._prevContent, {alpha:0, time:1.25, transition:TRANSITION_TYPE});	 
			if (this._prevContent != this._content) {
				Tweener.addTween(this._content, {alpha:1, time:1.25, transition:TRANSITION_TYPE, 
								 onStart:dispatchChangeEvent, onComplete:rewindPrevSWF});			
			}
			else {
				Tweener.addTween(this._content, {alpha:1, time:1.25, transition:TRANSITION_TYPE, 
								 onStart:dispatchChangeEvent});			
			}
			this.runCurrSWF();				 
		}
		
		//display slide in content
		private function slideInContent(index:uint):void {
			var moveBy:Number = -index * this._slideWidth;
			var transitionTime:Number = TRANSITION_TIME;
			this._prevContent = this._content;
			this._content = this._contents[index];
			
			if (moveBy == this._contentStrip.x) {
				transitionTime = 0.1;
			}
			Tweener.addTween(this._contentStrip, {x:moveBy, time:transitionTime, transition:TRANSITION_TYPE,
							onStart:dispatchChangeEvent, onComplete:runSWF});				
		}
		
		//dispatch content changed event		
		private function dispatchChangeEvent():void {
			dispatchEvent(new Event(CONTENT_CHANGE));	
		}
		
		//cleanup unused components
		private function cleanup():void {			
			if (this.numChildren > 3) {
				for (var i:uint = 0; i < this.numChildren; i++) {
					var child:Object = this.getChildAt(i);
					if (child != this._content && child != this._prevContent && child != this._maskSprite) {
						this.removeChildAt(i);
					}
				}
			}
		}
		
		//on slide click handler
		private function onSlideClick(event:MouseEvent):void {
			if (this._link && this._link != "") {		
				try {
					navigateToURL(new URLRequest(this._link), this._target);
				} 
				catch (e:Error) {
					trace("Cannot Load URL.");
				}	
			}
		}	
		
		//test for movie clip and play
		private function runSWF():void {
			this.rewindPrevSWF();
			this.runCurrSWF();			
		}
		
		private function runCurrSWF():void {
			try {	
				if (this._content is MovieClip) {
					MovieClip(this._content).gotoAndPlay(1);
				}
			}
			catch (e:Error) {
				trace(e.message);
			}							
		}
		
		private function rewindPrevSWF():void {
			try {	
				if (this._prevContent && this._prevContent is MovieClip) {
					MovieClip(this._prevContent).gotoAndStop(1);
				}		
			}
			catch (e:Error) {
				trace(e.message);
			}			
		}
		
		//on IO error handler
		private function ioError(event:IOErrorEvent):void {
			trace("IO Error: " + event.text);
			this._contents.push(new Sprite());
			
			this._loadIndex++;
			if (this._loadIndex < this._paths.length) {	
				//load next
				this.loadContent();
			}
			else {										
				//all complete
				this.setComplete();
			}	
		}
		
		//add path to list
		public function addPath(path:String):void {
			this._paths.push(path);
		}
		
		//set link
		public function set link(link:String):void {
			this._link = link;	
		}
		
		//set target
		public function set target(target:String):void {
			this._target = target;	
		}
		
		
	}
}