package com.webtako.flash {
	import caurina.transitions.Tweener;
	import caurina.transitions.properties.TextShortcuts;
	TextShortcuts.init();
	
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.net.URLRequest;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import flash.text.AntiAliasType;
	import flash.text.TextFormatAlign;
	import flash.display.DisplayObject;
	import flash.events.IOErrorEvent;
	
	public class Thumbnail extends Sprite {		
		public static const TRANSITION_TYPE:String = "easeOutQuart";
		public static const TRANSITION_TIME:Number = 1.0;
		public static const TEXT_FONT:MyFont = new MyFont();
		public static const THUMB_LOADED:String = "thumbLoaded";
		public static const DIM_VALUE:Number = -0.5;
		public static const ELLIPSE_SIZE:uint = 8;
		public static const BORDER_SIZE:uint = 1;
		public static const SQUARE:String = "SQUARE";
		public static const CIRCLE:String = "CIRCLE";
		public static const ROUND_SQUARE:String = "ROUND_SQUARE";
		
		private var _index:uint;
		private var _path:String;
		private var _link:String;
		private var _target:String;
		private var _description:String;
		private var _thumbnailSize:Number;
		private var _textField:TextField;
		private var _frameSprite:Sprite;		
		private var _tooltip:Tooltip;
		private var _loader:Loader;
		private var _container:Sprite;
		private var _color:uint;
		private var _mouseoverColor:uint;				
		private var _bgColor:uint;
		private var _mouseoverBgColor:uint;
		private var _frameColor:uint;
		private var _mouseoverFrameColor:uint;
		private var _selected:Boolean;		
		private var _radius:Number;
		
		public function Thumbnail(index:uint, caption:String, description:String, 
							  path:String, link:String, target:String, thumbnailSize:Number, textSize:Number = 12, 
							  color:uint = 0xFFFFFF, mouseoverColor:uint = 0x0066FF, 
							  bgColor:uint = 0x000000, mouseoverBgColor:uint = 0x0066FF, 
							  frameColor:uint = 0x333333, mouseoverFrameColor:uint = 0x0066FF, 
							  displayImage:Boolean = true, displayFrame:Boolean = true, shapeType:String = SQUARE,  
							  tooltipAlign:String = "BR", tooltipTextSize:Number = 12, 
							  tooltipColor:uint = 0xFFFFFF, tooltipBgColor:uint = 0x000000, 
							  tooltipAlpha:Number = 0.5, tooltipRoundCorners:Boolean = false) {									
			this.alpha = 0;
			
			//set properties
			this._index = index;
			this._description = description;
			this._path = path;
			this._link = link;
			this._target = target;
			this._thumbnailSize = thumbnailSize;
			this._color = color;
			this._mouseoverColor = mouseoverColor;
			this._bgColor = bgColor;
			this._mouseoverBgColor = mouseoverBgColor;
			this._frameColor = frameColor;
			this._mouseoverFrameColor = mouseoverFrameColor;
			this._selected = false;
			this._radius = Math.round(this._thumbnailSize/2);
			shapeType = shapeType.toUpperCase();
			
			//init frame
			if (displayFrame) {
				this.initFrame(shapeType);
			}
			
			//init background container
			this._container = new Sprite();
			this._container.graphics.beginFill(this._bgColor);	
			switch (shapeType) {
				case CIRCLE:
					this._container.graphics.drawCircle(this._radius, this._radius, this._radius);
					break;
				case ROUND_SQUARE:
					this._container.graphics.drawRoundRect(0, 0, this._thumbnailSize, this._thumbnailSize, ELLIPSE_SIZE, ELLIPSE_SIZE);
					break;
				default:
					this._container.graphics.drawRect(0, 0, this._thumbnailSize, this._thumbnailSize);
			}
			this._container.graphics.endFill();
			this.addChild(this._container);

			//init mask							
			this.initMask(shapeType);
		
			if (displayImage) {
				//init loader
				this._loader = new Loader();
				this._loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, ioError);
				this._loader.contentLoaderInfo.addEventListener(Event.COMPLETE, onLoadComplete);	
			}
			else {
				//init text panel
				this.initTextField((this._index + 1).toString(), textSize);
			}
			
			//init tooltip
			if (caption && caption != "") {
				this._tooltip = new Tooltip(caption, tooltipAlign, tooltipTextSize, 
											tooltipColor, tooltipBgColor, tooltipAlpha, tooltipRoundCorners);
			}
			
			//init mouse behaviors
			this.buttonMode = true;
			this.useHandCursor = true;	
			this.mouseEnabled = true;
			this.mouseChildren = false;			
								
			this.addEventListener(MouseEvent.MOUSE_OVER, onMouseover);
			this.addEventListener(MouseEvent.MOUSE_OUT,  onMouseout);
			this.addEventListener(MouseEvent.MOUSE_MOVE, onMousemove);
		}
		
		//init frame
		private function initFrame(type:String):void {			
			this._frameSprite = new Sprite();
			this._frameSprite.graphics.beginFill(this._frameColor);			
			switch (type) {
				case CIRCLE:
					this._frameSprite.graphics.drawCircle(this._radius, this._radius, this._radius);	
					break;
				case ROUND_SQUARE:					
					this._frameSprite.graphics.drawRoundRect(0, 0, this._thumbnailSize, this._thumbnailSize, ELLIPSE_SIZE, ELLIPSE_SIZE);
					break;
				default:
					this._frameSprite.graphics.drawRect(0, 0, this._thumbnailSize, this._thumbnailSize);				
			}				
			this._frameSprite.graphics.endFill();
			this.addChild(this._frameSprite);	
		}
			
		//init mask for container
		private function initMask(type:String):void {	
			var maskRadius:Number = this._radius;
			var maskSize:Number = this._thumbnailSize;
			var offset:uint = 0;
			
			if (this._frameSprite) {
				maskRadius -= BORDER_SIZE;
				maskSize -= (2 * BORDER_SIZE);
				offset = BORDER_SIZE;
			}
						
			var maskSprite:Sprite = new Sprite();
			maskSprite.graphics.beginFill(0x000000);
			switch (type) {
				case CIRCLE:
					maskSprite.graphics.drawCircle(maskRadius, maskRadius, maskRadius);
					break;
				case ROUND_SQUARE:					
					maskSprite.graphics.drawRoundRect(0, 0, maskSize, maskSize, ELLIPSE_SIZE, ELLIPSE_SIZE);
					break;
				default:
					maskSprite.graphics.drawRect(0, 0, maskSize, maskSize);
			}				
			maskSprite.graphics.endFill();
			
			maskSprite.x = offset;
			maskSprite.y = offset;	
			this._container.mask = maskSprite;	
			this._container.addChild(maskSprite);
		}
		
		//init text field panel
		private function initTextField(labelText:String, textSize:Number):void {
			//init text format
			var textFormat:TextFormat = new TextFormat();
			textFormat.font = TEXT_FONT.fontName;
			textFormat.size = textSize;
			textFormat.color = this._color;	
			textFormat.align = TextFormatAlign.CENTER;
			
			//init text field
			this._textField = new TextField();
			this._textField.defaultTextFormat = textFormat;
			this._textField.autoSize = TextFieldAutoSize.LEFT;
			this._textField.selectable = false;	
			this._textField.wordWrap = true;
			this._textField.embedFonts = true;
			this._textField.antiAliasType = AntiAliasType.ADVANCED;	
			this._textField.width = this._thumbnailSize;		
			this._textField.text = labelText;	
			
			this._textField.y = Math.round((this._thumbnailSize - this._textField.height)/2);
			this.addChild(this._textField);				
		}

		//on mouse over handler
		private function onMouseover(event:MouseEvent):void {
			if (!this._selected) {
				this.overChange();
			}
						
			//display tooltip
			if (this._tooltip) {
				this._tooltip.display(event);
			}
		}
		
		//on mouse out handler
		private function onMouseout(event:MouseEvent):void {
			if (!this._selected) {
				this.outChange();
			}
				
			//hide tooltip
			if (this._tooltip) {	
				this._tooltip.hide();
			}
		}
		
		//on mouse move handler
		private function onMousemove(event:MouseEvent):void {
			//move tooltip
			if (this._tooltip) {
				this._tooltip.move(event);			
			}
		}
		
		//highlight selected thumbnail
		public function highlight():void {
			this._selected = true;			
			this.overChange();
		}	
		
		//unhighlight thumbnail
		public function unhighlight():void {
			this._selected = false;
			this.outChange();			
		}	
		
		private function overChange():void {
			if (this._textField) {
				Tweener.addTween(this._container, {_color:this._mouseoverBgColor, time:TRANSITION_TIME, transition:TRANSITION_TYPE});		
				Tweener.addTween(this._textField, {_text_color:this._mouseoverColor, time:TRANSITION_TIME, transition:TRANSITION_TYPE});	
			}
			else {
				Tweener.addTween(this._container, {_brightness:DIM_VALUE, time:TRANSITION_TIME, transition:TRANSITION_TYPE});			
			}
			
			if (this._frameSprite) {
				Tweener.addTween(this._frameSprite, {_color:this._mouseoverFrameColor, time:TRANSITION_TIME, transition:TRANSITION_TYPE});
			}
		}
		
		private function outChange():void {
			if (this._textField) {
				Tweener.addTween(this._container, {_color:this._bgColor, time:TRANSITION_TIME, transition:TRANSITION_TYPE});		
				Tweener.addTween(this._textField, {_text_color:this._color, time:TRANSITION_TIME, transition:TRANSITION_TYPE});					
			}
			else {
				Tweener.addTween(this._container, {_brightness:0, time:TRANSITION_TIME, transition:TRANSITION_TYPE});						
			}
				
			if (this._frameSprite) {
				Tweener.addTween(this._frameSprite, {_color:this._frameColor, time:TRANSITION_TIME, transition:TRANSITION_TYPE});
			}		
		}
		
		//load content from path
		public function loadContent():void {
			if (this._loader) {
				this._loader.load(new URLRequest(this._path));	
			}
			else {
				Tweener.addTween(this, {alpha:1, transition:TRANSITION_TYPE, onComplete:dispatchLoadedEvent});		
			}
		}
		
		//on load complete handler
		private function onLoadComplete(event:Event):void {
			var content:DisplayObject = this._loader.content;
			content.x = Math.round((this._thumbnailSize - content.width)/2);
			content.y = Math.round((this._thumbnailSize - content.height)/2);
			this._container.addChild(content);
			
			//fade in content
			Tweener.addTween(this, {alpha:1, transition:TRANSITION_TYPE, onComplete:dispatchLoadedEvent});			
		}
		
		private function dispatchLoadedEvent():void {
			dispatchEvent(new Event(THUMB_LOADED));	
		}
		
		private function ioError(event:IOErrorEvent):void {
			trace("IO Error: " + event.text);	
			//fade in content
			Tweener.addTween(this, {alpha:1, transition:TRANSITION_TYPE, onComplete:dispatchLoadedEvent});
		}
		
		//get index
		public function get index():uint {
			return this._index;	
		}
		
		//get link
		public function get link():String {
			return this._link;	
		}
		
		//get target
		public function get target():String {
			return this._target;	
		}
		
		//get description
		public function get description():String {
			return this._description;	
		}
		
		//get tooltip
		public function get tooltip():Tooltip {
			return this._tooltip;
		}
	}
}