package com.webtako.flash {
	import caurina.transitions.Tweener;
	
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.text.AntiAliasType;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	public class Tooltip extends Sprite {
		public static const TEXT_FONT:MyFont = new MyFont();		
		public static const TRANSITION_TYPE:String = "easeOutQuart";
		public static const TRANSITION_TIME:Number = 0.25;
		public static const CURSOR_OFFSET:int = 12;		
		public static const ELLIPSE_SIZE:uint = 6;
		public static const TOP_LEFT:String     = "TL";
		public static const TOP_RIGHT:String    = "TR";
		public static const BOTTOM_LEFT:String  = "BL";
		public static const BOTTOM_RIGHT:String = "BR";
		
		private var _xOffset:Number;
		private var _yOffset:Number;
		
		public function Tooltip(message:String = "", align:String = TOP_LEFT, textSize:Number = 12, textColor:uint = 0xFFFFFF, 
								bgColor:uint = 0x000000, bgAlpha:Number = 0.5, roundCorners:Boolean = false) {
			//init text format
			var textFormat:TextFormat = new TextFormat();
			textFormat.font = TEXT_FONT.fontName;
			textFormat.size = textSize;
			textFormat.color = textColor;	
			
			//init text field
			var textField:TextField = new TextField();
			textField.defaultTextFormat = textFormat;
			textField.selectable = false;	
			textField.multiline = false;			
			textField.wordWrap = false;
			textField.embedFonts = true;
			textField.antiAliasType = AntiAliasType.ADVANCED;				
			textField.autoSize = TextFieldAutoSize.LEFT;
			textField.text = message;
			
			//init background	
			var container:Sprite = new Sprite();
			container.graphics.beginFill(bgColor, bgAlpha);
			if (roundCorners) {
				container.graphics.drawRoundRect(0, 0, textField.width, textField.height, ELLIPSE_SIZE, ELLIPSE_SIZE);
			}
			else {
				container.graphics.drawRect(0, 0, textField.width, textField.height);
			}
			container.graphics.endFill();
			container.addChild(textField);
			this.addChild(container);
			
			//set align offset
			align = align.toUpperCase();
			switch (align) {
				case TOP_LEFT:
					this._xOffset = -this.width;
					this._yOffset = -this.height;
					break;		
				case TOP_RIGHT:
					this._xOffset = CURSOR_OFFSET;
					this._yOffset = -this.height;
					break;		
				case BOTTOM_LEFT:
					this._xOffset = -(this.width + CURSOR_OFFSET);
					this._yOffset = CURSOR_OFFSET;
					break;
				default:	
					this._xOffset = CURSOR_OFFSET;
					this._yOffset = CURSOR_OFFSET;
			}
			
			//init misc. properties
			this.mouseChildren = false;
			this.mouseEnabled = false;
			
			this.visible = false;
			this.alpha = 0;
		}
		
		//display tooltip
		public function display(event:MouseEvent):void {			
			this.x = event.stageX + this._xOffset;
			this.y = event.stageY + this._yOffset;			
			
			Tweener.addTween(this, {alpha:1,time:TRANSITION_TIME, transition:TRANSITION_TYPE, onStart:setVisible});				
		}
		
		//mouse move event handler
		public function move(event:MouseEvent):void {
			this.x = event.stageX + this._xOffset;
			this.y = event.stageY + this._yOffset;			
			
			event.updateAfterEvent(); 	
		}
		
		//hide tooltip
		public function hide():void {
			Tweener.addTween(this, {alpha:0,time:TRANSITION_TIME, transition:TRANSITION_TYPE, onComplete:setInvisible});	
		}
		
		private function setVisible():void {
			this.visible = true;
		}
		
		private function setInvisible():void {
			this.visible = false;	
		}
		
		private function flip():void {
			
		}
	}
}